#!/usr/bin/env -S perl -T

use warnings;
use strict;

use File::Spec::Functions ( "catfile", "catdir", "file_name_is_absolute", "splitdir" );
use Scalar::Util ( "tainted" );

$ENV{PATH} = "/usr/local/bin:/usr/bin:/bin";

sub usage {
    my ( $stream ) = @_;
    $stream = *STDERR unless defined $stream;
    print $stream "backup-restore MACHINE DATE PATH\n"
}

if ( @ARGV < 3 ) {
    usage(*STDOUT);
    exit 1;
}

my $machine;
if ( (shift @ARGV) =~ m|^([^/ ]*)$| ) {
    $machine = $1;
} else {
    print STDERR "Machine must be a machine name only, not a path\n";
    usage;
    exit 2;
}
my $date;
if ( (shift @ARGV) =~ /([0-9]+-[0-9]+-[0-9]+)/ ) {
    $date = $1;
} else {
    print STDERR "Date must be in YEAR-MOONTH-DAY format\n";
    usage;
    exit 2;
}
my @paths = ();
for my $p ( @ARGV ) {
	# strip the leading / to match paths in the tar
    if ( $p =~ m|^/(.*)$| ) {
        push( @paths, $1 );
    } else {
        print STDERR "Expected absolute path but got $p\n";
        usage;
        exit 2;
    }
}

print "$machine $date @paths\n";

my $full;
my @weekly = ();
my @daily = ();

my $namematch = qr/^([0-9]+-[0-9]+-[0-9]+)[.](daily|weekly|full)[.](.*)$/;

opendir( my $dir, catdir("/backup-tar", $machine ) ) or die $!;
my @backups = sort( grep { /$namematch/ } readdir( $dir ) );
for my $backup ( @backups ) {
    $backup =~ /$namematch/;
    $backup = "$1.$2.$3";
    my $path = catfile( "/backup-tar", $machine, $backup );
    my $from = $1;
    my $type = $2;
    last if $from gt $date;

    if ( $type eq "full" ) {
        $full = $path;
        @weekly = ();
    } elsif ( $type eq "weekly" ) {
        push( @weekly, $path );
        @daily = ();
    } else {
        push( @daily, $path );
    }
}

my @to_restore = ( $full, @weekly, @daily );

for my $backup ( @to_restore ) {
    print "$backup…\n";
    my @args = ( "tar", "--extract", "--auto-compress", "--incremental", "--xattrs",
				 "--acls", "-vv", "-f$backup", "--", @paths );
	system( @args ) == 0 or die "extract of $backup failed with $?, @args\n";
}
